#!/usr/bin/env python3
import json
import os
import sys
from pathlib import Path

from sqlalchemy import (
    create_engine,
    text,
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

sys.path.append(str(Path(__file__).parent.parent))
from terminal_bench.config import config as terminal_bench_config
from terminal_bench.db import DBRunMetadata, DBTrialResult, get_db_url
from terminal_bench.harness_models import RunMetadata, TrialResults

# Create SQLAlchemy Base
Base = declarative_base()


def init_db():
    """Initialize the database schema."""
    try:
        # First connect to 'postgres' database to create our database if needed
        engine = create_engine(get_db_url("postgres"))
        with engine.connect() as conn:
            # Create database if it doesn't exist
            dbname = terminal_bench_config.db_name
            result = conn.execute(
                text(f"SELECT 1 FROM pg_catalog.pg_database WHERE datname = '{dbname}'")
            )
            exists = result.fetchone()

            if not exists:
                # We need to commit the transaction and run CREATE DATABASE
                conn.execute(text("commit"))
                print(f"Creating database {dbname}...")
                conn.execute(text(f'CREATE DATABASE "{dbname}"'))
                conn.commit()

        engine.dispose()

        # Now connect to our database and create tables
        engine = create_engine(get_db_url())
        Base.metadata.create_all(engine)
        print("Database schema created successfully.")
    except Exception as e:
        print(f"Error initializing database: {e}")
        sys.exit(1)


def import_run_data(output_dir="runs", run_id=None):
    """Import run data from the filesystem into the database.

    Args:
        output_dir: Directory containing the run data
        run_id: Specific run ID to import (if None, import all runs)
    """
    db_url = get_db_url()
    engine = create_engine(db_url)
    SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)
    session = SessionLocal()

    try:
        # Get all run directories
        run_dirs = [
            d
            for d in os.listdir(output_dir)
            if os.path.isdir(os.path.join(output_dir, d))
            and os.path.exists(os.path.join(output_dir, d, "results.json"))
            and (run_id is None or d == run_id)
        ]

        for run_dir in run_dirs:
            run_path = os.path.join(output_dir, run_dir)
            metadata_path = os.path.join(run_path, "run_metadata.json")
            results_path = os.path.join(run_path, "results.json")

            # Skip if files don't exist
            if not (os.path.exists(metadata_path) and os.path.exists(results_path)):
                print(f"Skipping {run_dir}: Missing metadata or results file")
                continue

            # Load metadata and results
            with open(metadata_path) as f:
                metadata_dict = json.load(f)
            with open(results_path) as f:
                results_dict = json.load(f)

            # Check if run already exists in database
            existing_run = (
                session.query(DBRunMetadata)
                .filter_by(uuid=metadata_dict.get("uuid"))
                .first()
            )
            if existing_run:
                print(
                    f"Skipping {run_dir}: "
                    f"Run with UUID {metadata_dict.get('uuid')} already exists"
                )
                continue

            # Convert to Pydantic models
            metadata = RunMetadata(**metadata_dict)
            # Create database records
            db_metadata = DBRunMetadata.from_pydantic(metadata)

            # Add to session
            session.add(db_metadata)

            # Add task results
            for task_result_dict in results_dict.get("results", []):
                task_result = TrialResults(**task_result_dict)
                db_task = DBTrialResult.from_pydantic(task_result, metadata.uuid)
                session.add(db_task)

            print(f"Imported run {run_dir} (UUID: {metadata.uuid})")

        # Commit all changes
        session.commit()
        print(f"Successfully imported {len(run_dirs)} runs.")

    except Exception as e:
        session.rollback()
        print(f"Error importing run data: {e}")

    finally:
        session.close()


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Initialize and populate the database")
    parser.add_argument(
        "--init", action="store_true", help="Initialize the database schema"
    )
    parser.add_argument(
        "--import-runs", action="store_true", help="Import run data from filesystem"
    )
    parser.add_argument(
        "--output-dir", default="runs", help="Directory containing run data"
    )
    parser.add_argument("--run-id", help="Specific run ID to import")

    args = parser.parse_args()

    if args.init:
        init_db()

    if args.import_runs:
        import_run_data(args.output_dir, args.run_id)

    if not (args.init or args.import_runs):
        parser.print_help()
