#!/usr/bin/env python3
import sys
from pathlib import Path
from typing import Any, Dict, Type

from sqlalchemy import inspect, text
from sqlalchemy.engine import create_engine
from sqlalchemy.orm import DeclarativeBase

sys.path.append(str(Path(__file__).parent.parent))
from terminal_bench.db import DBRunMetadata, DBTrialResult, get_db_url


def get_model_columns(model: Type[DeclarativeBase]) -> Dict[str, Any]:
    """Get column information from a SQLAlchemy model.

    Args:
        model: SQLAlchemy model class

    Returns:
        Dict mapping column names to their types
    """
    return {column.name: str(column.type) for column in model.__table__.columns}


def get_table_columns(engine: Any, table_name: str) -> Dict[str, str]:
    """Get existing column information from a database table.

    Args:
        engine: SQLAlchemy engine
        table_name: Name of the table

    Returns:
        Dict mapping column names to their types
    """
    inspector = inspect(engine)
    return {col["name"]: str(col["type"]) for col in inspector.get_columns(table_name)}


def run_migration(sql: str) -> None:
    """Run a database migration safely.

    Args:
        sql: SQL to run for the migration
    """
    db_url = get_db_url()
    engine = create_engine(db_url)

    try:
        with engine.begin() as conn:
            conn.execute(text(sql))
        print("Migration completed successfully")
    except Exception as e:
        print(f"Error during migration: {e}")
        sys.exit(1)
    finally:
        engine.dispose()


def modify_column(
    table_name: str,
    column_name: str,
    column_type: str,
    default_value: Any = None,
    nullable: bool = True,
    exists: bool = False,
) -> None:
    """Modify a column in a table safely. Can add a new column or alter an existing one.

    Args:
        table_name: Name of the table to modify
        column_name: Name of the column to modify
        column_type: SQL type for the column
        default_value: Default value for the column (if any)
        nullable: Whether the column can be NULL
        exists: Whether the column already exists (determines if we add or alter)
    """
    if exists:
        # Alter existing column
        sql_commands = [
            f"""
            ALTER TABLE {table_name}
            ALTER COLUMN {column_name} TYPE {column_type}
            USING {column_name}::text::{column_type}
            """
        ]

        if default_value is not None:
            sql_commands.append(
                f"""
                ALTER TABLE {table_name}
                ALTER COLUMN {column_name} SET DEFAULT {default_value}
                """
            )

        if not nullable:
            sql_commands.append(
                f"""
                ALTER TABLE {table_name}
                ALTER COLUMN {column_name} SET NOT NULL
                """
            )
        else:
            sql_commands.append(
                f"""
                ALTER TABLE {table_name}
                ALTER COLUMN {column_name} DROP NOT NULL
                """
            )

        # Join all commands with semicolons
        sql = ";\n".join(sql_commands)
    else:
        # Add new column
        sql = f"""
        ALTER TABLE {table_name}
        ADD COLUMN {column_name} {column_type}
        """
        if default_value is not None:
            sql += f" DEFAULT {default_value}"
        if not nullable:
            sql += " NOT NULL"

    run_migration(sql)


def drop_column(table_name: str, column_name: str) -> None:
    """Drop a column from a table.

    Args:
        table_name: Name of the table
        column_name: Name of the column to drop
    """
    sql = f"""
    ALTER TABLE {table_name}
    DROP COLUMN {column_name}
    """
    run_migration(sql)


def create_table(table_name: str, columns: Dict[str, str]) -> None:
    """Create a new table with the specified columns.

    Args:
        table_name: Name of the table to create
        columns: Dict mapping column names to their types
    """
    # Build column definitions
    column_defs = []
    for name, type_ in columns.items():
        column_defs.append(f"{name} {type_}")

    # Create table SQL
    sql = f"""
    CREATE TABLE {table_name} (
        {", ".join(column_defs)}
    )
    """
    run_migration(sql)


def sync_schema() -> None:
    """Sync database schema with Pydantic models."""
    db_url = get_db_url()
    engine = create_engine(db_url)

    try:
        # Check if tables exist
        inspector = inspect(engine)
        existing_tables = inspector.get_table_names()

        # Get columns from models
        run_metadata_model_columns = get_model_columns(DBRunMetadata)
        trial_results_model_columns = get_model_columns(DBTrialResult)

        # Create tables if they don't exist
        if "run_metadata" not in existing_tables:
            print("Creating run_metadata table...")
            create_table("run_metadata", run_metadata_model_columns)
            run_metadata_columns = run_metadata_model_columns
        else:
            run_metadata_columns = get_table_columns(engine, "run_metadata")

        if "trial_results" not in existing_tables:
            print("Creating trial_results table...")
            create_table("trial_results", trial_results_model_columns)
            trial_results_columns = trial_results_model_columns
        else:
            trial_results_columns = get_table_columns(engine, "trial_results")

        # Find missing columns and type changes
        missing_run_metadata = set(run_metadata_model_columns.keys()) - set(
            run_metadata_columns.keys()
        )
        missing_trial_results = set(trial_results_model_columns.keys()) - set(
            trial_results_columns.keys()
        )

        # Find columns that exist in DB but not in models
        extra_run_metadata = set(run_metadata_columns.keys()) - set(
            run_metadata_model_columns.keys()
        )
        extra_trial_results = set(trial_results_columns.keys()) - set(
            trial_results_model_columns.keys()
        )

        # Find type changes
        type_changes_run_metadata = {
            col: new_type
            for col, new_type in run_metadata_model_columns.items()
            if col in run_metadata_columns and run_metadata_columns[col] != new_type
        }
        type_changes_trial_results = {
            col: new_type
            for col, new_type in trial_results_model_columns.items()
            if col in trial_results_columns and trial_results_columns[col] != new_type
        }

        # Check for potential data loss from type changes
        if type_changes_run_metadata or type_changes_trial_results:
            print("\nWARNING: The following type changes may result in data loss:")
            for col, new_type in type_changes_run_metadata.items():
                print(f"  - {col}: {run_metadata_columns[col]} -> {new_type}")
            for col, new_type in type_changes_trial_results.items():
                print(f"  - {col}: {trial_results_columns[col]} -> {new_type}")

            response = input("\nDo you want to proceed? (y/N): ")
            if response.lower() != "y":
                print("Migration cancelled")
                return

        # Check for columns to drop
        if extra_run_metadata or extra_trial_results:
            print(
                "\nWARNING: The following columns exist in the database but not"
                " in the models:"
            )
            for col in extra_run_metadata:
                print(f"  - run_metadata.{col}")
            for col in extra_trial_results:
                print(f"  - trial_results.{col}")

            print(
                "\nTo drop a column, type its full name "
                "(e.g. 'run_metadata.column_name')"
            )
            print("Type 'done' when finished, or press Enter to skip dropping columns")

            while extra_run_metadata or extra_trial_results:
                response = input("\nColumn to drop (or 'done'): ").strip()
                if response.lower() == "done" or not response:
                    break

                try:
                    table, column = response.split(".")
                    if table == "run_metadata" and column in extra_run_metadata:
                        print(f"Dropping column {column} from run_metadata table...")
                        drop_column("run_metadata", column)
                        extra_run_metadata.remove(column)
                    elif table == "trial_results" and column in extra_trial_results:
                        print(f"Dropping column {column} from trial_results table...")
                        drop_column("trial_results", column)
                        extra_trial_results.remove(column)
                    else:
                        print(f"Invalid column name: {response}")
                except ValueError:
                    print("Invalid format. Use 'table.column' format")

        # Add missing columns
        for column in missing_run_metadata:
            print(f"Adding column {column} to run_metadata table...")
            modify_column(
                table_name="run_metadata",
                column_name=column,
                column_type=run_metadata_model_columns[column],
                nullable=True,  # Default to nullable for safety
                exists=False,
            )

        for column in missing_trial_results:
            print(f"Adding column {column} to trial_results table...")
            modify_column(
                table_name="trial_results",
                column_name=column,
                column_type=trial_results_model_columns[column],
                nullable=True,  # Default to nullable for safety
                exists=False,
            )

        # Update column types
        for column, new_type in type_changes_run_metadata.items():
            print(
                f"Updating type of column {column} in run_metadata table from "
                f"{run_metadata_columns[column]} to {new_type}..."
            )
            modify_column(
                table_name="run_metadata",
                column_name=column,
                column_type=new_type,
                exists=True,
            )

        for column, new_type in type_changes_trial_results.items():
            print(
                f"Updating type of column {column} in trial_results table from "
                f"{trial_results_columns[column]} to {new_type}..."
            )
            modify_column(
                table_name="trial_results",
                column_name=column,
                column_type=new_type,
                exists=True,
            )

        if not (
            missing_run_metadata
            or missing_trial_results
            or type_changes_run_metadata
            or type_changes_trial_results
            or extra_run_metadata
            or extra_trial_results
        ):
            print("Database schema is up to date with models")

    finally:
        engine.dispose()


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(description="Database migration tools")
    parser.add_argument(
        "--sync",
        action="store_true",
        help="Sync database schema with local models",
    )

    args = parser.parse_args()

    if args.sync:
        sync_schema()
    else:
        parser.print_help()
