import asyncio
import hashlib
import hmac
import json
import os
from typing import Any, Dict, List, Optional

import aiohttp
import discord  # type: ignore
import uvicorn  # type: ignore
import yaml
from discord.ext import commands  # type: ignore
from dotenv import load_dotenv  # type: ignore
from fastapi import FastAPI, Header, Request  # type: ignore

# Load environment variables
load_dotenv()

# Bot setup
intents = discord.Intents.default()
bot = commands.Bot(command_prefix="!", intents=intents)

# Get environment variables with validation
channel_id_str = os.getenv("DISCORD_CHANNEL_ID")
if not channel_id_str:
    raise ValueError("DISCORD_CHANNEL_ID environment variable is not set")
channel_id = int(channel_id_str)

github_pat = os.getenv("GITHUB_PAT")
if not github_pat:
    raise ValueError("GITHUB_PAT environment variable is not set")

github_repo = os.getenv("GITHUB_REPO")
if not github_repo:
    raise ValueError("GITHUB_REPO environment variable is not set")

# Debug mode for local testing
DEBUG_MODE = os.getenv("DEBUG_MODE", "false").lower() == "true"

# FastAPI app for webhook
app = FastAPI()


def verify_signature(payload_body: bytes, signature_header: Optional[str]) -> bool:
    """Verify that the payload was sent from GitHub by validating the signature."""
    # Skip verification in debug mode
    if DEBUG_MODE:
        print("Debug mode: Skipping signature verification")
        return True

    if not signature_header:
        print("No signature header provided")
        return False

    hash_object = hmac.new(
        github_pat.encode("utf-8"),  # type: ignore
        msg=payload_body,
        digestmod=hashlib.sha256,  # type: ignore
    )
    expected_signature = f"sha256={hash_object.hexdigest()}"
    return hmac.compare_digest(expected_signature, signature_header)


def is_task_file(filename: str) -> bool:
    """Check if a file is a task YAML file.

    Args:
        filename: The full path of the file within the repository

    Returns:
        bool: True if the file is named task.yaml, regardless of directory
    """
    if not filename:
        return False
    return os.path.basename(filename) == "task.yaml"


def parse_task_yaml(content: str) -> Dict[str, Any]:
    """Parse task YAML content."""
    try:
        return yaml.safe_load(content)
    except yaml.YAMLError as e:
        print(f"Error parsing YAML: {e}")
        return {}


def get_task_description(task_data: Dict[str, Any]) -> str:
    """Extract task description from the task data."""
    if "descriptions" in task_data and isinstance(task_data["descriptions"], list):
        for desc in task_data["descriptions"]:
            if isinstance(desc, dict) and "description" in desc:
                return desc["description"]
    return "No description available"


async def fetch_file_content(owner: str, repo: str, path: str, ref: str) -> str:
    """Fetch file content from GitHub API."""
    url = f"https://api.github.com/repos/{owner}/{repo}/contents/{path}?ref={ref}"
    headers = {
        "Authorization": f"token {github_pat}",
        "Accept": "application/vnd.github.v3+json",
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(url, headers=headers) as response:
            if response.status == 200:
                data = await response.json()
                import base64

                content = base64.b64decode(data["content"]).decode("utf-8")
                return content
            else:
                print(f"Error fetching file: {response.status}")
                return ""


async def fetch_pr_files(owner: str, repo: str, pr_number: int) -> List[Dict[str, Any]]:
    """Fetch files changed in a pull request."""
    url = f"https://api.github.com/repos/{owner}/{repo}/pulls/{pr_number}/files"
    headers = {
        "Authorization": f"token {github_pat}",
        "Accept": "application/vnd.github.v3+json",
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(url, headers=headers) as response:
            if response.status == 200:
                return await response.json()
            else:
                print(f"Error fetching PR files: {response.status}")
                return []


@bot.event
async def on_ready():
    print(f"{bot.user} has connected to Discord!")


@app.post("/webhook")
async def github_webhook(
    request: Request, x_hub_signature_256: Optional[str] = Header(None)
):
    print("Received GitHub webhook")
    # Get the raw body
    body = await request.body()

    # Verify the signature
    if not verify_signature(body, x_hub_signature_256):
        print("Invalid signature")
        return {"status": "error", "message": "Invalid signature"}

    # Parse the payload
    payload = json.loads(body)

    # Get the event type
    event_type = request.headers.get("X-GitHub-Event", "unknown")

    # Handle pull request merges
    if (
        event_type == "pull_request"
        and payload.get("action") == "closed"
        and payload.get("pull_request", {}).get("merged")
    ):
        pr = payload["pull_request"]
        repo = payload["repository"]
        repo_name = repo.get("full_name", "")
        owner, repo_name = repo_name.split("/", 1)
        base_branch = pr["base"]["ref"]
        pr_number = pr["number"]

        print(f"Processing merged PR #{pr_number}")

        # Fetch files changed in the PR
        files = await fetch_pr_files(owner, repo_name, pr_number)
        print(f"Found {len(files)} files in PR")
        # Check if any task files were added
        task_files = [f for f in files if is_task_file(f.get("filename", ""))]
        print(f"Found {len(task_files)} task files")

        if task_files:
            for file in task_files:
                filename = file.get("filename", "")
                print(f"Processing task file: {filename}")

                # Fetch file content from GitHub
                content = await fetch_file_content(
                    owner, repo_name, filename, base_branch
                )
                if not content:
                    print(f"Failed to fetch content for {filename}")
                    continue

                # Parse task YAML
                task_data = parse_task_yaml(content)
                if not task_data:
                    print(f"Failed to parse YAML for {filename}")
                    continue

                # Get task description
                description = get_task_description(task_data)
                # Truncate description if it's too long
                if len(description) > 1024:
                    description = description[:1021] + "..."

                # Create an embed message for the task
                embed = discord.Embed(
                    title=f"🎉 New Task Added: {filename}",
                    url=pr["html_url"],
                    color=discord.Color.green(),
                )

                # Add task information
                embed.add_field(name="Description", value=description, inline=False)

                # Add author email if available
                if "author_email" in task_data:
                    embed.add_field(
                        name="Author Email",
                        value=task_data["author_email"],
                        inline=True,
                    )

                # Add difficulty if available
                if "difficulty" in task_data:
                    embed.add_field(
                        name="Difficulty", value=task_data["difficulty"], inline=True
                    )

                # Add category if available
                if "category" in task_data:
                    embed.add_field(
                        name="Category", value=task_data["category"], inline=True
                    )

                # Add PR information
                embed.add_field(name="Repository", value=repo["full_name"], inline=True)
                embed.add_field(name="Author", value=pr["user"]["login"], inline=True)
                embed.add_field(name="Branch", value=pr["base"]["ref"], inline=True)

                # Send the message to the specified channel
                channel = bot.get_channel(channel_id)
                if channel:
                    await channel.send(embed=embed)  # type: ignore

    return {"status": "success"}


async def start_bot():
    """Start the Discord bot"""
    bot_token = os.getenv("DISCORD_BOT_TOKEN")
    if not bot_token:
        raise ValueError("DISCORD_BOT_TOKEN environment variable is not set")
    await bot.start(bot_token)


async def start_server():
    """Start the FastAPI server"""
    config = uvicorn.Config(app, host="0.0.0.0", port=8000, loop="asyncio")
    server = uvicorn.Server(config)
    await server.serve()


async def main():
    """Run both the Discord bot and the FastAPI server"""
    await asyncio.gather(start_bot(), start_server())


if __name__ == "__main__":
    asyncio.run(main())
