from unittest.mock import Mock

import pytest
from docker.models.containers import Container

from terminal_bench.terminal.tmux_session import TmuxSession


@pytest.fixture
def mock_container():
    container = Mock(spec=Container)
    container.exec_run.return_value = Mock(exit_code=0, output=b"mock output")
    return container


@pytest.fixture
def tmux_session(mock_container):
    return TmuxSession(session_name="test-session", container=mock_container)


def test_prepare_keys_single_string(tmux_session):
    """Test preparing a single string key."""
    result, is_blocking = tmux_session._prepare_keys("ls", block=False)
    assert result == ["ls"]
    assert is_blocking is False


def test_prepare_keys_list(tmux_session):
    """Test preparing a list of keys."""
    result, is_blocking = tmux_session._prepare_keys(["cd", "dir"], block=False)
    assert result == ["cd", "dir"]
    assert is_blocking is False


def test_prepare_keys_non_blocking_with_enter(tmux_session):
    """Test non-blocking keys ending with Enter."""
    result, is_blocking = tmux_session._prepare_keys(["ls", "Enter"], block=False)
    assert result == ["ls", "Enter"]
    assert is_blocking is False


def test_prepare_keys_blocking_with_enter(tmux_session):
    """Test blocking keys ending with Enter."""
    result, is_blocking = tmux_session._prepare_keys(["ls", "Enter"], block=True)
    assert result == ["ls", tmux_session._TMUX_COMPLETION_COMMAND, "Enter"]
    assert is_blocking is True


def test_prepare_keys_blocking_with_newline(tmux_session):
    """Test blocking keys ending with newline."""
    result, is_blocking = tmux_session._prepare_keys(["echo 'hello'\n"], block=True)
    assert result == ["echo 'hello'", tmux_session._TMUX_COMPLETION_COMMAND, "Enter"]
    assert is_blocking is True


def test_prepare_keys_multiple_enters(tmux_session):
    """Test handling multiple Enter keys in blocking mode."""
    result, is_blocking = tmux_session._prepare_keys(
        ["ls", "Enter", "Enter"], block=True
    )
    assert result == ["ls", "; tmux wait -S done", "Enter"]
    assert is_blocking is True


def test_prepare_keys_only_newlines(tmux_session):
    """Test handling input with only newlines in blocking mode."""
    result, is_blocking = tmux_session._prepare_keys(["\n", "\r\n"], block=True)
    assert result == [tmux_session._TMUX_COMPLETION_COMMAND, "Enter"]
    assert is_blocking is True


def test_prepare_keys_empty_list(tmux_session):
    """Test preparing an empty list of keys."""
    result, is_blocking = tmux_session._prepare_keys([], block=True)
    assert result == []
    assert is_blocking is False


def test_prepare_keys_blocking_without_enter(tmux_session):
    """Test preparing a blocking command without an Enter key."""
    result, is_blocking = tmux_session._prepare_keys(["ls"], block=True)
    assert result == ["ls"]
    assert is_blocking is False


def test_prepare_keys_mixed_newlines_and_content(tmux_session):
    """Test handling mixed content with newlines in blocking mode."""
    result, is_blocking = tmux_session._prepare_keys(
        ["echo 'hello'\n", "world\n"], block=True
    )
    assert result == ["echo 'hello'\n", "world", "; tmux wait -S done", "Enter"]
    assert is_blocking is True


def test_prepare_keys_carriage_return(tmux_session):
    """Test handling carriage returns in blocking mode."""
    result, is_blocking = tmux_session._prepare_keys(["echo 'hello'\r"], block=True)
    assert result == ["echo 'hello'", "; tmux wait -S done", "Enter"]
    assert is_blocking is True
